//--------------------------------------------------------------------------
// File and Version Information:
//   Time-stamp: <01/09/01 04:16:47 yuji>
// Description:
//     TEST Module
//
//
// Environment:
//	Software developed for the CDF Detector.
//
// Author List:
//
// revision history:
// -----------------
//------------------------------------------------------------------------

#ifndef _SvxMyModule_HH_
#define _SvxMyModule_HH_

//----------------------
// Base Class Headers --
//----------------------
#include <string>
#include "BaBar/Cdf.hh"
class AbsEvent;

#include "Framework/APPModule.hh"
class StorableSiPedInfoSet;

#include "Framework/AbsParmBool.hh"
#include "Framework/AbsParmString.hh"
#include "Framework/AbsParmGeneral.hh"
#include "Framework/AbsParmFilename.hh"

#include "SiliconGeometry/SiDigiCode.hh"
#include "SvxCalibDBTables/SVXDh.Defs.hh"
#include "SvxCalibDBTables/SVXDh.hh"

class SiVrbInfoSet;

class SvxMyModule : public AppModule {
//--------------------
// Instance Members --
//--------------------

public:
  // Constructors
  SvxMyModule(const char* const theName="SvxMyModule",
	      const char* const theDescription=
                 "Test Module for Silicon");

  // Destructor
  virtual ~SvxMyModule() {};

  // Operations
  virtual AppResult beginJob(AbsEvent* anEvent);
  virtual AppResult event   (AbsEvent* anEvent);
  virtual AppResult beginRun(AbsEvent* anEvent);
  virtual AppResult endRun  (AbsEvent* anEvent);
  virtual AppResult endJob  (AbsEvent* anEvent);
  virtual AppResult abortJob(AbsEvent* anEvent);
  virtual AppModule* clone(const char* cloneName);

protected:

private:
  typedef AbsParmGeneral<int> AbsParmInt;

  ////////////////////////////////////
  // Helper functions

  // Talk-to parameters
  AbsParmInt _debugFirst;
  AbsParmInt _debugLast;
  AbsParmInt _anaBarrelStart;
  AbsParmInt _anaBarrelEnd;
  AbsParmInt _anaHalfLadderStart;
  AbsParmInt _anaHalfLadderEnd;
  AbsParmInt _anaPhiWedgeStart;
  AbsParmInt _anaPhiWedgeEnd;
  AbsParmInt _anaLayerStart;
  AbsParmInt _anaLayerEnd;
  AbsParmInt _anaLadderStart; 
  AbsParmInt _anaLadderEnd;
  AbsParmInt _anaSideStart;
  AbsParmInt _anaSideEnd;

  AbsParmBool _createStripSet;
  AbsParmBool _testStripSet;
  AbsParmBool _listStrips;
  AbsParmString _stripSetDescription;

  ////////////////////////////////////
  // Data

  int _eventN;
  bool _debug;
  bool _runBegan;
  bool _runEnded;
  bool _eventSeen;

  SiVrbInfoSet* _svxDaqInfo;
  SiVrbInfoSet* _islDaqInfo;

};

#endif

