//--------------------------------------------------------------------------
// File and Version Information:
//
// Description:
//    This module displays various information
//
// Environment:
//
// Author List:
//
// Copyright Information:
//
//------------------------------------------------------------------------

//---------------
// C++ Headers --
//---------------
#ifdef DEFECT_OLD_IOSTREAM_HEADERS
#include <iostream.h>
#include <ios.h>
#include <iomanip.h>
#else
#include <iostream>
#include <ios>
#include <iomanip>
#endif

#include <vector>
#ifndef DEFECT_NO_NAMESPACES
#  ifndef DEFECT_NO_IOSTREAM_NAMESPACES
using std::cout;
using std::endl;
using std::ifstream;
using std::ios;
using std::dec;
using std::hex;
#  endif 
#endif
//-----------------------
// This Class's Header --
//-----------------------
#include "MyMods/SvxMyModule.hh"

//-------------------------------
// Collaborating Class Headers --
//-------------------------------
#include "AbsEnv/AbsEnv.hh"

#ifdef USE_CDFEDM2
#include "Edm/ConstHandle.hh"
#include "TrackingObjects/Storable/StorableRun2SiStripSet.hh"
#endif
#include "SiliconGeometry/AbsSiNumerology.hh"
#include "SiliconGeometry/HWSiNumerology.hh"
#include "CalibDB/RunListKey.hh"
#include "SvxDaqUtils/SiDReadVec.hh"

//-----------------------------------------------------------------------
// Local Macros, Typedefs, Structures, Unions and Forward Declarations --
//-----------------------------------------------------------------------

//		----------------------------------------
// 		-- Public Function Member Definitions --
//		----------------------------------------

//----------------
// Constructors --
//----------------

SvxMyModule::SvxMyModule(const char* const theName, 
                           const char* const theDescription)
  : AppModule(theName, theDescription )
  , _debugFirst("Debug_first",this,1)
  , _debugLast("Debug_last",this,3)
  , _anaBarrelStart("anaBarrelStart",this,0)
  , _anaBarrelEnd("anaBarrelEnd",this, HWSiNumerology::N_BARRELS_MAX-1)
  , _anaHalfLadderStart("anaHalfLadderStart",this,0)
  , _anaHalfLadderEnd("anaHalfLadderEnd",this,2-1)
  , _anaPhiWedgeStart("anaPhiWedgeStart",this,0)
  , _anaPhiWedgeEnd("anaPhiWedgeEnd",this, HWSiNumerology::N_PHI_MAX-1)
  , _anaLayerStart("anaLayerStart",this,0)
  , _anaLayerEnd("anaLayerEnd",this, HWSiNumerology::N_LAYERS_MAX-1)
  , _anaLadderStart("anaLadderStart",this,0)
  , _anaLadderEnd("anaLadderEnd",this,0)
  , _anaSideStart("anaSideStart",this,0)
  , _anaSideEnd("anaSideEnd",this, HWSiNumerology::N_READOUT_SIDES_MAX-1)
  , _listStrips("listStrips", this, false)
  , _createStripSet("createStripSet", this, false)
  , _testStripSet("testStripSet", this, false)
  , _stripSetDescription( "stripSetDescription", this, "MyModule" )
  , _debug(false)
  , _runBegan(false)
  , _runEnded(false)
  , _eventSeen(false)
{
#ifdef USE_CDFEDM2
  _debugFirst.addDescription( string(
    "\t\t\tFirst event to debug:Debug_first  set <n>" ));
  _debugLast.addDescription( string(
    "\t\t\tLast  event to debug:Debug_last   set <n>"));
  _anaBarrelStart.addDescription( string(
    "\t\t\tFirst Barrel to Analyze  set <n>"));
  _anaBarrelEnd.addDescription( string(
    "\t\t\tLast Barrel to Analyze  set <n>"));
  _anaHalfLadderStart.addDescription( string(
    "\t\t\tFirst HalfLadder to Analyze  set <n>"));
  _anaHalfLadderEnd.addDescription( string(
    "\t\t\tLast HalfLadder to Analyze  set <n>"));
  _anaPhiWedgeStart.addDescription( string(
    "\t\t\tFirst PhiWedge to Analyze  set <n>"));
  _anaPhiWedgeEnd.addDescription( string(
    "\t\t\tLast PhiWedge to Analyze  set <n>"));
  _anaLayerStart.addDescription( string(
    "\t\t\tFirst Layer to Analyze  set <n>"));
  _anaLayerEnd.addDescription( string(
    "\t\t\tLast Layer to Analyze  set <n>"));
  _anaLadderStart.addDescription( string(
    "\t\t\tFirst Ladder to Analyze  set <n>"));
  _anaLadderEnd.addDescription( string(
    "\t\t\tLast Ladder to Analyze  set <n>"));
  _anaSideStart.addDescription( string(
    "\t\t\tFirst Side to Analyze  set <n>"));
  _anaSideEnd.addDescription( string(
    "\t\t\tLast Side to Analyze  set <n>"));
  _listStrips.addDescription( string(
    "\t\t\tList Strips in StorableRun2SiStripSet" ));
  _createStripSet.addDescription( string(
    "\t\t\tCreate StorableRun2SiStripSet in EventRecord" ));
  _testStripSet.addDescription( string(
    "\t\t\ttest StorableRun2SiStripSet in EventRecord" ));
  _stripSetDescription.addDescription( string(
    "\t\t\tDescription for the SiStripSet produced" ));

  commands()->append(&_debugFirst);
  commands()->append(&_debugLast);
  commands()->append(&_anaBarrelStart);
  commands()->append(&_anaBarrelEnd);
  commands()->append(&_anaHalfLadderStart);
  commands()->append(&_anaHalfLadderEnd);
  commands()->append(&_anaPhiWedgeStart);
  commands()->append(&_anaPhiWedgeEnd);
  commands()->append(&_anaLayerStart);
  commands()->append(&_anaLayerEnd);
  commands()->append(&_anaLadderEnd);
  commands()->append(&_anaLadderEnd);
  commands()->append(&_anaSideStart);
  commands()->append(&_anaSideEnd);
  commands()->append(&_listStrips);
  commands()->append(&_createStripSet);
  commands()->append(&_testStripSet);
  commands()->append(&_stripSetDescription);

#endif // USE_CDFEDM2
}

//--------------
// Operations --
//--------------

/*
 *
 *  beginJob
 *
 */
AppResult SvxMyModule::beginJob( AbsEvent* anEvent ) {
  _eventN = 0;
  _runBegan=true;
  _runEnded=false;
  _eventSeen=false;

  // set up the Daq Info Sets
  SiVrbInfoSet::DetType detType;
  _svxDaqInfo = new SiVrbInfoSet();
  detType = SiVrbInfoSet::SVXII;
  _svxDaqInfo->daqInfoSetUp(detType);
  _islDaqInfo = new SiVrbInfoSet();
  detType = SiVrbInfoSet::ISL;
  _islDaqInfo->daqInfoSetUp(detType);
  cout << name() << ": SvxPedModule: set up daqinfo set" << endl;

  cout << name() << ": Begin Job Successful" << endl;
  return AppResult::OK;
}

/*
 *
 *  endJob
 *
 */
AppResult SvxMyModule::endJob( AbsEvent* anEvent ) {
  cout << name() << ": End Job Successful" << endl;
  return AppResult::OK;
}

/*
 *
 *  beginRun
 *
 */
AppResult SvxMyModule::beginRun( AbsEvent* anEvent ) {
  cout << name() << ": Run# " << gblEnv->runNumber() << endl;
  cout << name() << ": Begin Run Successful" << endl;
  return AppResult::OK;
}

/*
 *
 *  abortJob
 *
 */
AppResult SvxMyModule::abortJob( AbsEvent* anEvent ) {
  cout << name() << ": Abort Job Successful" << endl;
  return AppResult::OK;
}

/*
 *
 *  endRun
 *
 */
AppResult SvxMyModule::endRun( AbsEvent* anEvent ) {
  _runBegan=false;
  _runEnded=true;
  _eventSeen=false;
  cout << name() << ": End Job Successful" << endl;
  return AppResult::OK;
}


/*
 *
 *  clone
 *
 */
AppModule* SvxMyModule::clone( const char* cloneName ) {
  return new SvxMyModule( cloneName,
			  "This module is a clone of SvxMyModule." );
}


/*
 *
 *  event
 *
 */
AppResult SvxMyModule::event( AbsEvent* anEvent){
  if(!_runBegan){
    cout << "Event seen but no begin run --> call begin run"<<endl;
    AppResult stat=beginRun(anEvent);
    if(stat!=AppResult::OK){
      return stat;
    }
  }
  _eventSeen=true;
  ++_eventN; // keep track of how many events we've processed
  _debug = (_eventN >= _debugFirst.value() && _eventN <= _debugLast.value());
  if (_debug) 
    cout << name() << "::event: physics event " << _eventN << endl;

  // createStripSet
  //   if createStripSet parameter is set, create StorableRun2SiStripSet
  //   and then, entry it to the eventRecord.
  if (_createStripSet.value()) {
    StorableRun2SiStripSet *ps=0;
    const SiStripSet *myStrips = ps = new StorableRun2SiStripSet(anEvent);
    // Probably these line below may not be necessary.
    //    GeometricRegion & region = TWorld;
    //    ps->loadFromBanks(anEvent, region );

    //    ps->writeBanks( anEvent );

    Handle<StorableRun2SiStripSet> h(ps);
    h->set_description( _stripSetDescription.value() );
    anEvent->append(h);
  }

  // testStripSet
  //   if testStripSet parameter is set, test if StorableRun2SiStripSet
  //   exist in the eventRecord and put the result on display.
  if (_testStripSet.value()) {
    EventRecord::ConstIterator iter(anEvent, "StorableRun2SiStripSet" );
    cout << name() << ": StorableRun2SiStripSet ConstIterator - "
	 << iter.is_valid() << endl;
  }

  // listStrips
  //   if listStrips is set, scan all strip if StorableRun2SiStripSet exist,
  //   and list all strip exist in the eventRecord.
  // 28-Sep-2000
  //   attempt new iteration frame
  if (_listStrips.value()) {
    EventRecord::ConstIterator iter(anEvent, "StorableRun2SiStripSet" );
    if( iter.is_valid() ) {
      /* // is equivalent to below
      const SiStripSet *myStrips=
	(ConstHandle<StorableRun2SiStripSet>(iter)).operator->();
	*/
      ConstHandle<StorableRun2SiStripSet> myStrips(iter);
      cout << "Now iterating over all strips" << endl;

      // iterate over all the strips.
      // Loop over barrel, halfLadder, phiWedge, layer, ladder and side.
      // -- What is ladder? It seems to be completely dummy.
      for( unsigned phiWedge=_anaPhiWedgeStart.value();
	   phiWedge<=_anaPhiWedgeEnd.value(); phiWedge++ )
	for( unsigned barrel=_anaBarrelStart.value();
	     barrel<=_anaBarrelEnd.value(); barrel++ )
	  for( unsigned halfLadder=_anaHalfLadderStart.value();
	     halfLadder<=_anaHalfLadderEnd.value(); halfLadder++ )
	    for( unsigned layer=_anaLayerStart.value();
		 layer<=_anaLayerEnd.value(); layer++ )
	      for( unsigned ladder=_anaLadderStart.value();
		   ladder<=_anaLadderEnd.value(); ladder++ )
		for( unsigned side=_anaSideStart.value();
		     side<=_anaSideEnd.value(); side++ ) {
		  // set SiDigiCode of this ladder.
		  SiDigiCode detector( barrel, halfLadder, phiWedge,
				       layer, ladder, side);
		  // check validity of this ladder.
		  if( myStrips->nStrips(detector) <= 0 ) {
		    // might be invalid, skip this ladder.
		    continue;
		  }

		  // initialization before iteration over strips here.
		  unsigned minStrip(9999), maxStrip(0);

		  // This for-loop is
		  // iteration over all strips in this ladder.
		  for (SiStripSet::ConstIterator
			 s=myStrips->beginDetector(detector);
		       s!=myStrips->endDetector(detector); ++s) {

		    // from now on, we describe iterated task.
		    if( minStrip>s->getStripNumber() )
		      minStrip=s->getStripNumber();
		    if( maxStrip<s->getStripNumber() )
		      maxStrip=s->getStripNumber();

		  } // end of loop for strips

		  // end task of iteration over strips here.
		  cout << name() << ": B:" << detector.getBarrel()
		       << " HL:" << detector.getHalfLadder()
		       << " PW:" << detector.getPhiWedge()
		       << " L:" << detector.getLayer()
		       << " S:" << detector.getPnSide()
		       << " strip:" << minStrip << " - " << maxStrip
		       << " (" << myStrips->nStrips(detector) << ")"
		       <<endl;

		} // end of loop for side, ... and barrel
    } // correspond to if( iter.is_valid() )
    else {
      cout << name() << ": StorableRun2SiStripSet is invalid." << endl;
    }
  }
  return AppResult::OK;
}

/* EOF */
